<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of scrapingbot
 *
 * @author scrapingbot
 */

if (!defined('_CAN_LOAD_FILES_'))
    exit;

include_once dirname(__FILE__).'/classes/ScrapingbotPlatform.php';
include_once dirname(__FILE__).'/classes/ScrapingbotPrice.php';
include_once dirname(__FILE__).'/classes/ScrapingbotFileLogger.php';

class ScrapingBot extends Module {

    public static $logger;
    //construct
    public function __construct() {
        $this->name = 'scrapingbot';
        $this->tab = 'front_office_features';
        $this->version = '1.2';
        $this->author = 'Scrapingbot';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Price monitoring on competitor\'s products made quick and easy for you');
        $this->description = $this->l('Scrape and extract data from any product page without getting blocked !');

        $this->ps_versions_compliancy = array('min' => '1.6.0.0', 'max' => _PS_VERSION_);
        
        $updateAvailable = $this->checkForUpdates();

        // Check if an update is available
        if($updateAvailable){
            // Display a warning in the back office
            $this->warning = $this->l('A new version of ScrapingBot module is available');
        }
    }

    //check for updates
    public function checkForUpdates(){
        $currentVersion = $this->version;

        // Get the remote version
        $url = 'https://www.scraping-bot.io/prestashop_version/';
        $remoteVersion = Tools::file_get_contents($url);

        // Check if the remote version is greater than the current version
        if($remoteVersion){
            if(version_compare($remoteVersion, $currentVersion) > 0){
                return true;
            }
        }
        return false;
    }

    //install
    public function install() {
        return parent::install()
            && $this->_installSQL()
            && $this->_installTab()
            && Configuration::updateValue('SCRAPINGBOT_USER', '')
            && Configuration::updateValue('SCRAPINGBOT_KEY', '')
            && Configuration::updateValue('SCRAPINGBOT_ORIGIN', 'PrestaShop')
            && Configuration::updateValue('SCRAPINGBOT_EXPEDITOR', Configuration::get('PS_SHOP_EMAIL'))
            && Configuration::updateValue('SCRAPINGBOT_DISPLAY_PRODUCT', 0)
            && Configuration::updateValue('SCRAPINGBOT_DISPLAY_PRODUCT_NB', 5)
            && Configuration::updateValue('SCRAPINGBOT_SUPERPROXY', 1)
            && Configuration::updateValue('SCRAPINGBOT_COLLECTE_PRODUCT_NB', 20)
            && Configuration::updateValue('SCRAPINGBOT_DELAY_TO_COLLECTE', 24)
            && Configuration::updateValue('SCRAPINGBOT_PLAN', 'Free Plan')
            && Configuration::updateValue('SCRAPINGBOT_NBCREDIT', '0')
            && Configuration::updateValue('SCRAPINGBOT_NBCREDIT_FROM', '100')
            && Configuration::updateValue('SCRAPINGBOT_ENABLE_LOGS', '1')
            && Configuration::updateValue('SCRAPINGBOT_CRON_TOKEN', Tools::getToken())
            && Configuration::updateValue('SCRAPINGBOT_URL_API', 'http://api.scraping-bot.io/scrape')
            && Configuration::updateValue('SCRAPINGBOT_MAIL_TO', 'support@scraping-bot.io')
            && $this->registerHook('displayHeader')
            && $this->registerHook('header')
            && $this->registerHook('displayFooterProduct')
            && $this->registerHook('displayProductButtons')
            && $this->registerHook('displayLeftColumnProduct')
            && $this->registerHook('displayRightColumnProduct')
            && $this->registerHook('displayAdminProductsExtra')
            && $this->registerHook('actionObjectProductUpdateBefore')
            && $this->registerHook('actionObjectProductDeleteAfter');
    }

    //install SQL
    public function _installSQL(){
        $sql = 'CREATE TABLE IF NOT EXISTS  `'._DB_PREFIX_.'scrapingbot_platform` (
          `id_platform` int(11) NOT NULL AUTO_INCREMENT,
          `id_shop` INT NOT NULL,
          `name` varchar(100) CHARACTER SET utf8 NOT NULL,
          `url` varchar(250) CHARACTER SET utf8 NOT NULL,
          `image` varchar(250) CHARACTER SET utf8 NOT NULL,
          `active` tinyint(1) NOT NULL,
          `position` int(11) NOT NULL,
           PRIMARY KEY (`id_platform`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';
        $res = Db::getInstance()->execute($sql);

        $sql2 = 'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'scrapingbot_product` (
          `id_scrapingbot` int(11) NOT NULL AUTO_INCREMENT,
          `id_platform` int(11) NOT NULL,
          `id_product` int(11) NOT NULL,
          `url` varchar(250) CHARACTER SET utf8 NOT NULL,
          `date_update` datetime NOT NULL,
          `active` tinyint(1) NOT NULL,
          PRIMARY KEY (`id_scrapingbot`),
          UNIQUE KEY `id_platform` (`id_platform`,`id_product`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';
        $res2 = Db::getInstance()->execute($sql2);

        $sql3 = 'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'scrapingbot_price` (
          `id_price` int(11) NOT NULL AUTO_INCREMENT,
          `id_platform` int(11) NOT NULL,
          `id_product` int(11) NOT NULL,
          `price` decimal(20,6) NOT NULL,
          `description` text NOT NULL,
          `shipping_price` decimal(20,6) NOT NULL,
          `image` VARCHAR(255) NOT NULL,
          `title` VARCHAR(255) NOT NULL,
          `data` TEXT NOT NULL,
          `date_add` datetime NOT NULL,
          PRIMARY KEY (`id_price`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';
        $res3 = Db::getInstance()->execute($sql3);

        return $res && $res2 && $res3;
    }

    public function _installTab()
    {
        if (!$id_tab = Tab::getIdFromClassName('AdminScraping')) {
            $id_parent = (int)Tab::getIdFromClassName('AdminCatalog');
            // Install tab
            $tab = new Tab();
            $tab->name = array();
            foreach (Language::getLanguages() as $language){
                if($language['iso_code']=='FR_fr'){
                    $tab->name[$language['id_lang']] = 'Scrapingbot Prix';
                }else {
                    $tab->name[$language['id_lang']] = 'Scrapingbot Price';
                }
            }
            $tab->class_name = 'AdminScraping';
            $tab->id_parent = $id_parent;
            $tab->module = $this->name;

            if (!$tab->add())
                return false;
        }
        return true;
    }
    public function uninstall()
    {
        return parent::uninstall()
            && $this->_uninstallTab();
    }

    public function _uninstallTab()
    {
        $id_tab = (int)Tab::getIdFromClassName('AdminScraping');

        if ($id_tab) {
            $tab = new Tab($id_tab);
            $tab->delete();
        }

        return parent::uninstall();
    }

    public function getContent()
    {
        $this->_html = '';

        $updateAvailable = $this->checkForUpdates();
        if ($updateAvailable) {
            $this->_html .= $this->displayWarning($this->l('A new version of ScrapingBot module is available.'));
        }

        // If we try to update the settings
        if (isset($_POST['submitScrapingBotConfig']) || Tools::isSubmit('submitEditPlatform'))
        {
            $this->_html .= $this->postProcess();
        }
        if(Tools::isSubmit('statusscrapingbot_platform')){
            Db::getInstance()->execute('UPDATE '._DB_PREFIX_.'scrapingbot_platform SET active = IF (`active`, 0, 1) WHERE id_platform='.(int)Tools::getValue('id_platform'));
        }
        $userName= Configuration::get('SCRAPINGBOT_USER');
        $apiKey= Configuration::get('SCRAPINGBOT_KEY');

        if($userName!='' || $apiKey!='') {

            $plan = $this->getPlan();
            if ($plan) {
                Configuration::updateValue('SCRAPINGBOT_PLAN', $plan->planName);
                Configuration::updateValue('SCRAPINGBOT_NBCREDIT', $plan->apiCallsConsumed);
                Configuration::updateValue('SCRAPINGBOT_NBCREDIT_FROM', $plan->apiCallsLimit);
            }
        }

        $this->context->smarty->assign(array(
            'lang' => $this->context->language->iso_code,
            'module_dir' => $this->_path,
            'scrapingbot_version' => $this->version,
            'plan' => Configuration::get('SCRAPINGBOT_PLAN'),
            'credit' => Configuration::get('SCRAPINGBOT_NBCREDIT'),
            'credit_from' => Configuration::get('SCRAPINGBOT_NBCREDIT_FROM')
        ));
        $this->_html .= $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        AdminController::$currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        if (Tools::isSubmit('updatescrapingbot_platform'))
            $this->_html .= $this->renderForm((int)Tools::getValue('id_platform'));
        else
        {
            $this->url_simplified = $this->context->link->getModuleLink($this->name,'crontab', array('token' =>  Configuration::get('SCRAPINGBOT_CRON_TOKEN')));
            $this->url = '*/15 * * * * curl -L -k "'.$this->context->link->getModuleLink($this->name,'crontab', array('token' =>  Configuration::get('SCRAPINGBOT_CRON_TOKEN'))).'" > /dev/null';
            $this->line_url = '*/15 * * * * php '.dirname(__FILE__).'/crontab.php token='.Configuration::get('SCRAPINGBOT_CRON_TOKEN').' > /dev/null';
            $this->_html .= $this->renderFormConfig().
                $this->renderList();
        }

        return $this->_html;
    }

    protected function renderFormConfig()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitScrapingBotConfig';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );
        return $helper->generateForm(array(
            array(
                'form' => $this->getConfigForm()
            )
        ));
    }

    protected function getConfigForm()
    {

        $form = array(
            'legend' => array(
                'title' => $this->l('Scraping-bot').' V'.$this->version,
                'icon' => 'icon-cogs',
            )
        );

        //======================================================================
        // CREDENTIALS TAB
        $form['tabs']['parameters'] = $this->l('Parameters');


        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'text',
            'label' => $this->l('ScrapingBot Username'),
            'name' => 'SCRAPINGBOT_USER',
            'required' => true,
        );

        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'text',
            'label' => $this->l('ScrapingBot API Key'),
            'name' => 'SCRAPINGBOT_KEY',
            'desc' => ' <a href="#" onclick="getPlan();return false;" class="btn button button-large" >'.$this->l('Check your connection').'</a></strong>',
        );
        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'hidden',
            'label' => $this->l('Url'),
            'name' => 'URL_CONTROLER',
            'required' => true,
        );

        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'switch',
            'label' => $this->l('Display on product page'),
            'name' => 'SCRAPINGBOT_DISPLAY_PRODUCT',
            'required' => false,
            'class' => 't',
            'is_bool' => true,
            'hint' => $this->l('Display the higher prices of the platforms in a comparative table on the product sheet '),
            'values' => array(
                array(
                    'id' => 'SCRAPINGBOT_DISPLAY_PRODUCT_on',
                    'value' => 1,
                    'label' => $this->l('Enabled')
                ),
                array(
                    'id' => 'SCRAPINGBOT_DISPLAY_PRODUCT_off',
                    'value' => 0,
                    'label' => $this->l('Disabled')
                )
            ),
        );

        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'select',
            'label' => $this->l('Choosing display  hook'),
            'name' => 'SCRAPINGBOT_HOOK',
            'options' => array(
                'query' => array(
                    array(
                        'id' => 'displayFooterProduct',
                        'name' => 'displayFooterProduct'
                    ),
                    array(
                        'id' => 'displayProductButtons',
                        'name' => 'displayProductButtons'
                    ),
                    array(
                        'id' => 'displayLeftColumnProduct',
                        'name' => 'displayLeftColumnProduct'
                    ),
                    array(
                        'id' => 'displayRightColumnProduct',
                        'name' => 'displayRightColumnProduct'
                    )
                ),
                'id' => 'id',
                'name' => 'name',
            )
        );

        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'text',
            'label' => $this->l('Number of platforms to display'),
            'name' => 'SCRAPINGBOT_DISPLAY_PRODUCT_NB',
            'required' => false,
        );

        $form['input'][] = array(
            'tab' => 'parameters',
            'type' => 'switch',
            'label' => $this->l('Allow super proxies'),
            'name' => 'SCRAPINGBOT_SUPERPROXY',
            'required' => false,
            'class' => 't',
            'is_bool' => true,
            'hint' => $this->l('Allow price recovery on platforms such as Amazon which are more complex.'),
            'desc' => $this->l('Some platforms such as Amazon are complex to analyze to retrieve prices, this cost is then 10 credits.'),
            'values' => array(
                array(
                    'id' => 'SCRAPINGBOT_SUPERPROXY_on',
                    'value' => 1,
                    'label' => $this->l('Enabled')
                ),
                array(
                    'id' => 'SCRAPINGBOT_SUPERPROXY_off',
                    'value' => 0,
                    'label' => $this->l('Disabled')
                )
            )
        );

        //======================================================================
        // GENERAL TAB

        $form['tabs']['advanced'] = $this->l('Advanced');


        $form['input'][] = array(
            'tab' => 'advanced',
            'type' => 'text',
            'label' => $this->l('Email expeditor'),
            'name' => 'SCRAPINGBOT_EXPEDITOR',
            'required' => true,
        );
        $form['input'][] = array(
            'tab' => 'advanced',
            'type' => 'text',
            'label' => $this->l('Number product by cron'),
            'name' => 'SCRAPINGBOT_COLLECTE_PRODUCT_NB',
            'required' => true,
        );
        $form['input'][] = array(
            'tab' => 'advanced',
            'type' => 'text',
            'label' => $this->l('Delay to update product'),
            'name' => 'SCRAPINGBOT_DELAY_TO_COLLECTE',
            'prefix' => $this->l('hours'),
            'hint' => $this->l('The recovery of a price for a product will be done every ').Configuration::get('SCRAPINGBOT_DELAY_TO_COLLECTE').$this->l('hours.'),
            'required' => true,
        );
        $form['input'][] = array(
            'tab' => 'advanced',
            'type' => 'switch',
            'label' => $this->l('Enabled logs'),
            'name' => 'SCRAPINGBOT_ENABLE_LOGS',
            'required' => false,
            'class' => 't',
            'is_bool' => true,
            'values' => array(
                array(
                    'id' => 'SCRAPINGBOT_ENABLE_LOGS_on',
                    'value' => 1,
                    'label' => $this->l('Enabled')
                ),
                array(
                    'id' => 'SCRAPINGBOT_ENABLE_LOGS_off',
                    'value' => 0,
                    'label' => $this->l('Disabled')
                )
            )
        );

        //======================================================================
        // SYSTEM TAB
        $form['tabs']['crontab'] = $this->l('Cron');

        $form['input'][] = array(
            'tab' => 'crontab',
            'type' => 'free',
            'label' => $this->l('Cron url'),
            'name' => 'URL_SIMPLIFIED',
            'desc' => $this->l('we recommend one call every 15 minutes'),
        );
        $form['input'][] = array(
            'tab' => 'crontab',
            'type' => 'free',
            'name' => 'CRON_NOTE',
            'desc' => '<hr/><strong>'.$this->l('Exemple to call').
                ' <a href="https://www.scraping-bot.io/prestashop-addon/" target="_blank" >'.$this->l('view in FAQ').'</a></strong><ul>'.
                '<li>'.$this->l('With the call of a URL').'</li>'.
                '<li>'.$this->l('With the command line call to be added in your crontab').'</li></ul>',
        );
        $form['input'][] = array(
            'tab' => 'crontab',
            'type' => 'free',
            'label' => $this->l('Command with url'),
            'name' => 'URL',
        );
        $form['input'][] = array(
            'tab' => 'crontab',
            'type' => 'free',
            'label' => $this->l('Command line (recommanded)'),
            'name' => 'LINE_URL',
        );

        $form['submit'] = array(
            'title' => $this->l('Save')
        );

        return $form;
    }

    public function renderList()
    {
        $scraptinbot_platform = ScrapingbotPlatform::getPlatforms();
        $fields_list = array(
            'id_platform' => array(
                'title' => $this->l('ID'),
                'align' => 'text-center',
                'class' => 'fixed-width-xs'
            ),
            'name' => array(
                'title' => $this->l('Name'),
            ),
            'url' => array(
                'title' => $this->l('Url to scrape'),
            ),
            'active' => array(
                'title' => $this->l('Enabled'),
                'active' => 'status',
                'filter_key' => 'a!active',
                'align' => 'center',
                'type' => 'bool',
                'orderby' => false,
                'class' => 'fixed-width-sm'
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->actions = array('edit', 'delete');
        $helper->show_toolbar = true;
        $helper->toolbar_btn['new'] = array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&module_name='.$this->name.'&updatescrapingbot_platform',
            'desc' => $this->l('Create a platform', null, null, false)
        );
        $helper->module = $this;
        $helper->identifier = 'id_platform';
        $helper->orderBy = 'id_platform';
        $helper->orderWay = 'ASC';
        $helper->title = $this->l('List of platforms');
        $helper->table = 'scrapingbot_platform';
        $helper->listTotal = count($scraptinbot_platform);
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;


        return $helper->generateList($scraptinbot_platform, $fields_list);
    }

    function renderForm($id_platform = 0)
    {
        if($id_platform>0){
            $platform = new ScrapingbotPlatform($id_platform);
        }
        $fields_form_1 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Create a platform'),
                    'icon' => 'icon-cogs',
                    'tinymce' => true,
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_platform',
                    ),
                    array(
                        'type' => 'hidden',
                        'name' => 'id_shop',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Name'),
                        'name' => 'name',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Url'),
                        'name' => 'url',
                        'required' => true,
                    ),
                    array(
                        'type' => 'file',
                        'label' => $this->l('Image'),
                        'name' => 'image',
                        'thumb' => isset($platform) ? '../modules/'.$this->name.'/images/'.$platform->image : '',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Active'),
                        'name' => 'active',
                        'required' => true,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                    'name' => 'submitEditPlatform',
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table =  $this->name;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEditPlatform';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getBlockFieldsValues($id_platform),
        );

        return $helper->generateForm(array($fields_form_1));
    }

    public function getBlockFieldsValues($id = 0)
    {
        $platform = new ScrapingbotPlatform($id);

        return array(
            'id_platform' => $platform->id,
            'name' => $platform->name,
            'url' => $platform->url,
            'active' => $platform->active,
            'image' => $platform->image,
            'id_shop' => $platform->id_shop
        );
    }
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $return = array();
        $return['SCRAPINGBOT_USER'] = Configuration::get('SCRAPINGBOT_USER',Tools::getValue('SCRAPINGBOT_USER'));
        $return['SCRAPINGBOT_KEY'] = Configuration::get('SCRAPINGBOT_KEY', Tools::getValue('SCRAPINGBOT_KEY'));
        $return['SCRAPINGBOT_ENABLE_LOGS'] = Configuration::get('SCRAPINGBOT_ENABLE_LOGS', Tools::getValue('SCRAPINGBOT_ENABLE_LOGS'));
        $return['SCRAPINGBOT_DISPLAY_PRODUCT'] = (bool)Configuration::get('SCRAPINGBOT_DISPLAY_PRODUCT');
        $return['SCRAPINGBOT_EXPEDITOR'] = Configuration::get('SCRAPINGBOT_EXPEDITOR');
        $return['SCRAPINGBOT_DISPLAY_PRODUCT_NB'] = (int)Configuration::get('SCRAPINGBOT_DISPLAY_PRODUCT_NB');
        $return['SCRAPINGBOT_COLLECTE_PRODUCT_NB'] = (int)Configuration::get('SCRAPINGBOT_COLLECTE_PRODUCT_NB');
        $return['SCRAPINGBOT_DELAY_TO_COLLECTE'] = (int)Configuration::get('SCRAPINGBOT_DELAY_TO_COLLECTE',Tools::getValue('SCRAPINGBOT_DISPLAY_PRODUCT_NB'));
        $return['SCRAPINGBOT_SUPERPROXY'] = (bool)Configuration::get('SCRAPINGBOT_SUPERPROXY');
        $return['SCRAPINGBOT_HOOK'] = Configuration::get('SCRAPINGBOT_HOOK');
        $return['CRON_NOTE'] = '';
        $return['URL_CONTROLER'] = $this->context->link->getModuleLink($this->name, 'ajax');
        $return['URL_SIMPLIFIED'] = '<p class="form-control-static"><a href="'.htmlentities($this->url_simplified, ENT_NOQUOTES, 'UTF-8').'" target="_blank">'.htmlentities($this->url_simplified, ENT_NOQUOTES, 'UTF-8').'</a></p>';
        $return['URL'] = '<p class="form-control-static">'.htmlentities($this->url, ENT_NOQUOTES, 'UTF-8').'</p>';
        $return['LINE_URL'] = '<p class="form-control-static">'.htmlentities($this->line_url, ENT_NOQUOTES, 'UTF-8').'</p>';

        return $return;
    }

    public function postProcess(){
        if(Tools::isSubmit('submitScrapingBotConfig')) {
            Configuration::updateValue('SCRAPINGBOT_USER', Tools::getValue('SCRAPINGBOT_USER'));
            Configuration::updateValue('SCRAPINGBOT_KEY', Tools::getValue('SCRAPINGBOT_KEY'));
            Configuration::updateValue('SCRAPINGBOT_ENABLE_LOGS', (bool)Tools::getValue('SCRAPINGBOT_ENABLE_LOGS'));
            Configuration::updateValue('SCRAPINGBOT_DISPLAY_PRODUCT', (bool)Tools::getValue('SCRAPINGBOT_DISPLAY_PRODUCT'));
            Configuration::updateValue('SCRAPINGBOT_DISPLAY_PRODUCT_NB', (int)Tools::getValue('SCRAPINGBOT_DISPLAY_PRODUCT_NB'));
            Configuration::updateValue('SCRAPINGBOT_EXPEDITOR', Tools::getValue('SCRAPINGBOT_EXPEDITOR'));
            Configuration::updateValue('SCRAPINGBOT_COLLECTE_PRODUCT_NB', (int)Tools::getValue('SCRAPINGBOT_COLLECTE_PRODUCT_NB'));
            Configuration::updateValue('SCRAPINGBOT_DELAY_TO_COLLECTE', (int)Tools::getValue('SCRAPINGBOT_DELAY_TO_COLLECTE'));
            Configuration::updateValue('SCRAPINGBOT_SUPERPROXY', (bool)Tools::getValue('SCRAPINGBOT_SUPERPROXY'));
            Configuration::updateValue('SCRAPINGBOT_HOOK', Tools::getValue('SCRAPINGBOT_HOOK'));
            return $this->displayConfirmation($this->l('Configuration updated'));
        }
        if(Tools::isSubmit('submitEditPlatform')){
            $id_platform = (int)Tools::getValue('id_platform');
            $platform = new ScrapingbotPlatform($id_platform);
            $image = $platform->image;
            $platform->copyFromPost($platform, 'scrapingbot_platform');

            /* Uploads image and sets slide */
            $type = strtolower(substr(strrchr($_FILES['image']['name'], '.'), 1));
            $imagesize = @getimagesize($_FILES['image']['tmp_name']);
            if (isset($_FILES['image']) &&
                isset($_FILES['image']['tmp_name']) &&
                !empty($_FILES['image']['tmp_name']) && $_FILES['image']['tmp_name']!='' &&
                !empty($imagesize) &&
                in_array(strtolower(substr(strrchr($imagesize['mime'], '/'), 1)), array('jpg', 'gif', 'jpeg', 'png')) &&
                in_array($type, array('jpg', 'gif', 'jpeg', 'png')))
            {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = sha1(microtime());
                if ($error = ImageManager::validateUpload($_FILES['image']))
                    $errors[] = $error;
                elseif (!$temp_name || !move_uploaded_file($_FILES['image']['tmp_name'], $temp_name))
                    return false;
                elseif (!ImageManager::resize($temp_name, dirname(__FILE__).'/images/'.Tools::encrypt($_FILES['image']['name'].$salt).'.'.$type, null, null, $type))
                    $errors[] = $this->displayError($this->l('An error occurred during the image upload process.'));
                if (isset($temp_name))
                    @unlink($temp_name);
                $platform->image = Tools::encrypt($_FILES['image']['name'].$salt).'.'.$type;
            }else{
                $platform->image = $image;
            }
            $platform->id_shop = isset($this->context->shop->id) ? $this->context->shop->id : 0;
            if ($platform->save())
                Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminModules').'&configure='.$this->name.'&conf=4');
            else
                $this->_html .= '<div class="conf confirm alert alert-danger">'.$this->l('the platform was unable to save').'</div>';
        }
    }

    //add form in product admin
    public function hookDisplayAdminProductsExtra($params){
        $id_product = Tools::getvalue('id_product', isset($params['id_product']) ? (int)$params['id_product'] : null);
        if (Validate::isLoadedObject($product = new Product($id_product, false, $this->context->language->id))) {

            $scraptinbot_platform = ScrapingbotPlatform::getPlatforms();

            if($scraptinbot_platform){
                $tabplatform = array();
                foreach($scraptinbot_platform as $key => $platform){
                    $tabplatform[$key]['id'] = $platform['id_platform'];
                    $tabplatform[$key]['name'] = $platform['name'];
                    $tabplatform[$key]['url'] = '';
                    $scraptinbot_url = Db::getInstance()->getValue('SELECT url FROM '._DB_PREFIX_.'scrapingbot_product WHERE id_product='.$product->id.' AND id_platform='.$platform['id_platform']);
                    if($scraptinbot_url!=''){
                        $tabplatform[$key]['url'] = $scraptinbot_url;
                    }
                    $tabplatform[$key]['date'] = '-';
                    $tabplatform[$key]['price'] = '-';
                    $tabplatform[$key]['description'] = '';
                    $tabplatform[$key]['shipping_price'] = '-';
                    $scraptinbot_price = Db::getInstance()->getRow('SELECT price, date_add, description, title, shipping_price, image FROM '._DB_PREFIX_.'scrapingbot_price WHERE id_product='.$product->id.' AND id_platform='.$platform['id_platform'].' ORDER BY date_add DESC');
                    if($scraptinbot_price!='' && is_array($scraptinbot_price)){
                        $tabplatform[$key]['title'] = $scraptinbot_price['title'];
                        $tabplatform[$key]['image'] = $scraptinbot_price['image'];
                        $tabplatform[$key]['date'] = Tools::displayDate($scraptinbot_price['date_add'], null, true);
                        $tabplatform[$key]['price'] = Tools::displayPrice($scraptinbot_price['price']);
                        $tabplatform[$key]['description'] = $scraptinbot_price['description'];
                        $tabplatform[$key]['shipping_price'] = Tools::displayPrice($scraptinbot_price['shipping_price']);;
                    }
                }
            }
            $this->context->smarty->assign(array(
                'version' => _PS_VERSION_,
                'id_product' => $id_product,
                'platforms' => $tabplatform,
                'controller' => $this->context->link->getModuleLink($this->name, 'ajax', array(), true)
            ));

            return $this->display(__FILE__, 'views/templates/admin/inproduct.tpl');
        } else {
            return $this->displayError($this->l('You must save this product before adding tabs'));
        }
    }

    public function hookActionObjectProductUpdateBefore($params){
        $id_product = (int)$params['object']->id;

        $scraptinbot_platform = ScrapingbotPlatform::getPlatforms();
        if(isset($_POST['product_url'])){
            foreach($scraptinbot_platform as $platform) {
                if(isset($_POST['product_url'][$platform['id_platform']])) {
                    if($_POST['product_url'][$platform['id_platform']]!='') {
                        Db::getInstance()->execute('REPLACE INTO '._DB_PREFIX_.'scrapingbot_product SET  
                            id_platform='.$platform['id_platform'].', 
                            id_product='.$id_product.', 
                            url="'.$_POST['product_url'][$platform['id_platform']].'"');
                    }else{
                        Db::getInstance()->execute('DELETE FROM '._DB_PREFIX_.'scrapingbot_product WHERE id_product='.$id_product.' AND id_platform='.$platform['id_platform']);
                        Db::getInstance()->execute('DELETE FROM '._DB_PREFIX_.'scrapingbot_price WHERE id_product='.$id_product.' AND id_platform='.$platform['id_platform']);
                    }

                }
            }
        }
    }

    public function hookActionObjectProductDeleteAfter($params) {
        $id_product = (int)$params['object']->id;
        if($id_product>0) {
            Db::getInstance()->execute('DELETE FROM ' . _DB_PREFIX_ . 'scrapingbot_product WHERE id_product=' . $id_product);
            Db::getInstance()->execute('DELETE FROM ' . _DB_PREFIX_ . 'scrapingbot_price WHERE id_product=' . $id_product);
        }
    }

    public function hookDisplayHeader($params) {
    	return $this->hookHeader($params);
    }

    public function hookHeader($params) {
	    if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
		    $this->context->controller->registerStylesheet('modules-' . $this->name . '-style', 'modules/' . $this->name . '/views/css/front.css', ['media' => 'all', 'priority' => 150]);
	    } else {
		    $this->context->controller->addCSS($this->_path.'views/css/front.css', 'all');
	    }
    }

    public function hookDisplayFooterProduct($params) {
        if(Configuration::get('SCRAPINGBOT_HOOK')=='displayFooterProduct'){
	        return $this->_prepareHook($params);
        }
    }

    public function hookDisplayProductButtons($params) {
        if(Configuration::get('SCRAPINGBOT_HOOK')=='displayProductButtons'){
	        return $this->_prepareHook($params);
        }
    }

    public function hookDisplayLeftColumnProduct($params) {
        if(Configuration::get('SCRAPINGBOT_HOOK')=='displayLeftColumnProduct'){
	        return $this->_prepareHook($params);
        }
    }

    public function hookDisplayRightColumnProduct($params) {
        if(Configuration::get('SCRAPINGBOT_HOOK')=='displayRightColumnProduct'){
	        return $this->_prepareHook($params);
        }
    }

    public function _prepareHook($params){
        if((int)Configuration::get('SCRAPINGBOT_DISPLAY_PRODUCT')==1){
            $id_product = isset($params['product']->id) ? $params['product']->id : Tools::getValue('id_product');
            $nb_plateform = (int)Configuration::get('SCRAPINGBOT_DISPLAY_PRODUCT_NB');
            if($nb_plateform==0) $nb_plateform = 5;
            $sql = 'SELECT pl.name, sp.price, pl.image, sp.shipping_price, MAX(sp.date_add) as date 
                FROM '._DB_PREFIX_.'scrapingbot_price sp 
                INNER JOIN '._DB_PREFIX_.'scrapingbot_platform pl ON sp.id_platform=pl.id_platform 
                WHERE sp.id_product='.$id_product.' AND pl.active=1 
                GROUP BY pl.name, sp.price, sp.shipping_price, pl.image LIMIT 0,'.$nb_plateform;

            $result = Db::getInstance()->executeS($sql);

            if(is_array($result) && count($result)>0) {
                foreach($result as &$rst){
                    if(file_exists(dirname(__FILE__).'/images/'.$rst['image'])){
                        $rst['url_image'] = $this->context->link->getBaseLink().'modules/'.$this->name.'/images/'.$rst['image'];
                        $rst['price'] = tools::displayPrice($rst['price']);
                    }
                }
                $this->context->smarty->assign(array(
                    'platforms' => $result
                ));

	            if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
		            return $this->fetch('module:scrapingbot/views/templates/hook/scrapingbot_17.tpl');
	            } else {
		            return $this->display(dirname(__FILE__), 'scrapingbot_16.tpl');
	            }
            }
        }
    }

    public function CallApi($id_product, $id_platform, $url, $unique=true){
        $userName= Configuration::get('SCRAPINGBOT_USER');
        $apiKey= Configuration::get('SCRAPINGBOT_KEY');
        $urlapi = Configuration::get('SCRAPINGBOT_URL_API');

        if($userName=='' || $apiKey=='' || $urlapi==''){
            return $this->displayError($this->l('the module is not configured'));
        }

        $auth = base64_encode($userName.":".$apiKey);

        $postParams = array(
            "url" => $url,
            'options' => array(
                "getHtml" => false,
                "useChrome" => false,
                "purpose" => "prestashop",
                "premiumProxy" => (bool)Configuration::get('SCRAPINGBOT_SUPERPROXY') // better for Amazon, linkedin consumes 10 calls if true
            )
        );
        $json = json_encode($postParams);

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => $urlapi,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $json,
            CURLOPT_HTTPHEADER => array(
                "Authorization: Basic ".$auth,
                "Content-Type: application/json"
            ),
        ));

        $responses = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        $result = array(
            'data' => array(),
            'errors' => array()
        );

        if ($err) {
            $result['errors'][] = "cURL Error #:" . $err;
        } else {
            $response = json_decode($responses);
            if(!$response){
                $result['errors'][] = $responses;
            }else if(isset($response->error) && $response->error!=''){
                $result['errors'][] = $response->error;
            }else{
                //enregistrement du prix
                $data = $response->data;
                if(isset($data->price) && (float)$data->price>0) {
                    $scrapingbot_price = new ScrapingbotPrice();
                    $scrapingbot_price->id_platform = $id_platform;
                    $scrapingbot_price->id_product = $id_product;
                    $scrapingbot_price->price = $data->price;
                    $scrapingbot_price->shipping_price = $data->shippingFees;
                    $scrapingbot_price->description = $data->description;
                    $scrapingbot_price->image = $data->image;
                    $scrapingbot_price->title = $data->title;
                    $scrapingbot_price->data = json_encode($data);
                    if ($scrapingbot_price->save()) {
                        $result['data']['date'] = Tools::displayDate($scrapingbot_price->date_add, null, true);
                        $result['data']['price'] = Tools::displayPrice((float)$scrapingbot_price->price);
                        $result['data']['title'] = $data->title;
                        $result['data']['shipping_price'] = Tools::displayPrice((float)$scrapingbot_price->shipping_price);
                        $result['data']['description'] = $data->description;
                        $result['data']['image'] = $data->image;
                        Db::getInstance()->execute('UPDATE ' . _DB_PREFIX_ . 'scrapingbot_product SET date_update=NOW() WHERE id_product=' . $id_product . ' AND id_platform=' . $id_platform);
                    } else {
                        $result['errors'][] = $this->l('Cannot save data');
                    }
                }else{
                    $result['errors'][] = $this->l('No data collected');
                    $result['button_mail'] = true;
                }
            }
        }
        if($unique) {
            die(json_encode($result));
        }else{
            return $result;
        }
    }
    public function CallCron(){
        $delay = (int)configuration::get('SCRAPINGBOT_DELAY_TO_COLLECTE', 24);
        $nburls = configuration::get('SCRAPINGBOT_COLLECTE_PRODUCT_NB', 20);
        $date = date('Y-m-d H:i:s');
        $dateValable = date('Y-m-d H:i:s',strtotime('-'.$delay.' hour',strtotime($date)));
        $logger = self::getLogger();
        //boucle sur les urls
        $urls = Db::getInstance()->executeS('SELECT id_product, id_platform, url FROM '._DB_PREFIX_.'scrapingbot_product WHERE date_update<"'.$dateValable.'" ORDER BY date_update DESC LIMIT 0,'.$nburls);
        if($urls && is_array($urls)){
            foreach($urls as $url){
                $return = $this->CallApi($url['id_product'], $url['id_platform'], $url['url'], false);
                if($return['errors']){
                    $logger->logError("Errors on call url : ".$url['url']." (id_product: ".$url['id_product'].", id_platform: ".$url['id_platform'].") Errors: ".print_r($return, true));
                }
            }
        }

        return 'checked';
    }
    public function SendEmail($id_product, $id_platform, $url){
        $to = Configuration::get('SCRAPINGBOT_MAIL_TO', 'support@scraping-bot.io');
        $from = Configuration::get('SCRAPINGBOT_EXPEDITOR');
        $data = Db::getInstance()->getValue('SELECT `data` FROM '._DB_PREFIX_.'scrapingbot_price 
                WHERE id_product='.$id_product.' AND id_platform='.$id_platform.' ORDER BY date_add DESC');
        $template_vars = array(
            'username' => Configuration::get('SCRAPINGBOT_USER'),
            'email' => $from,
            'url' => $url,
            'data' => $data
        );

        $return =  Mail::Send(
            $this->context->language->id,
            'error_url',
            Mail::l('PRESTASHOP SUPPORT', $this->context->language->id),
            $template_vars,
            $to,
            'Support Scraping-bot',
            $from,
            null,
            null,
            null,
            dirname(__FILE__).'/mails/',
            false,
            $this->context->shop->id);

        if($return) {
            die(json_encode(array('send'=> (int)$return)));
        }
    }

    public static function getLogger()
    {
        if (!self::$logger) {
            self::$logger = new ScrapingbotFileLogger(Configuration::get('SCRAPINGBOT_ENABLE_LOGS') != 'False');

            $logs_dir = _PS_ROOT_DIR_.'/var/logs/';
            if (!file_exists($logs_dir)) {
                $logs_dir = _PS_ROOT_DIR_.'/app/logs/';
                if (!file_exists($logs_dir)) {
                    $logs_dir = _PS_ROOT_DIR_.'/log/';
                }
            }

            self::$logger->setFilename($logs_dir.date('Y_m').'_scrapingbot.log');
        }

        return self::$logger;
    }

    public function getPlan($ajax=false, $user=null, $key=null){
        $userName= $user!='' ? $user :  Configuration::get('SCRAPINGBOT_USER');
        $apiKey= $key!='' ? $key : Configuration::get('SCRAPINGBOT_KEY');
        $urlapi = 'http://api.scraping-bot.io/scrapingbot/get-user-info';

        if($userName=='' || $apiKey==''){
            return $this->displayError($this->l('the module is not configured'));
        }else{
            if($user!='') Configuration::updateValue('SCRAPINGBOT_USER', $user);
            if($key!='') Configuration::updateValue('SCRAPINGBOT_KEY', $key);
        }

        $content = file_get_contents($urlapi.'?username='.$userName.'&password='.$apiKey);
        $result = json_decode($content);

        if($ajax) {
            die(json_encode($result));
        }else{
            return $result;
        }
    }
}
